/////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	Moises Melendez, sesowm@yahoo.com, julio 2016
//
//	Aplicacion ESP8266-BOLT que implementa un servidor TCP/IP desde el sistema BOLT PIC18F2550 usando 
//	comandos AT, se debe previamente conectar el mdulo ESP8266 a una red wi-fi mediante los comandos 
//	AT necesarios, se puede utilizar el sistema BOLT para esta funcion.
//
//	El programa se comunica con el mdulo ESP8266 mediante el puerto serie a 115200 bauds, esta es la 
//	velocidad predeterminada para la comunicacin con el mdulo, se pude cambiar mediante comandos AT
//	si se requere de otra velocidad de funcionamiento.
//
//	Las conexiones son como se muestran:
//
//					  ____________________________
//					 |  						  |
//	 ________________|__________		 _________|__________________		 _______________________
//	|				 |		    |		| 	   	PIN 9				 |	    |DB9  SISTEMA BOLT
//	|			  RX    VCC---+-------|3.3V					PIN 8|------| 2	   TX_PIC	
//	|ESP8266  GPIO 0    RESET |	  	|   	  MAX3232 y		PIN 7|------| 3	   RX_PIC
//	|		  GPIO 2    CH_PD-+-------|3.3V	REGULADOR 3.3V    GND|------| 5	   GND
//	|			 GND    TX    |		|			    		  +5V|------| 7	   +5V
//	|________________|__|_______|		|__PIN 10____GND_____________|		|
//					 |  |					 |  	  |						|
//					 |  |____________________|		  |						|
//					 |								  |
//					 |________________________________|						
//
//
//		Nota: Los numero de pin dentro del bloque MAX3232 y REGULADOR 3.3V corresponden al nmero
//	de pin del circuito integrado MAX3232, este circuito integrado debe alimentarse con 3.3 volts para 
//	obtener los 3.3 volts a partir de la alimentacin de 5 volts del sistema Bolt se debe utilIzar un 
// 	circuito integrado regulador de voltaje, por ejemplo AMS1117-3.3 o KA78R33
//
// 	Mientras se realiza el proceso de crear el servidor TCP/IP, se muestra en la pantalla LCD el efecto 
//	de cada uno de los comandos AT enviados al mdulo ESP8266, si se crea el servidor, un cliente re-
//	moto puede conectarse a esta servidor mediante el uso de la IP publica y el puerto de acceso 
//	directo al exterior (direc port forwarding).
//
//	Cuando el cliente se conecat al servidor creado el el sistema BOLT-ESP8266 puede enviar alguno 
//	de los siguientes comandos
//
// 	Para encender el relevador: enciende relevador
// 	Para apagar el relevador: apaga relevador
// 	Para leer la temperatura: lee temperatura
//
//	El servidor respondera con la funcion realizada o con un mensaje de comando no reconocido.
//
//	Debido a que no se puede apagar un servidor desde un cliente, el servidor se desconecta cuando 
// 	se apaga el sistema Bolt y el modulo Esp8266
//
//////////////////////////////////////////////////////////////////////////////////////////////////////
//																									//
#define SERVER_PORT			   "2026"		// Numero de puerto utilizado por el servidor			//	
//																									//
//////////////////////////////////////////////////////////////////////////////////////////////////////

#include <p18cxxx.h>
#include <delays.h>
#include <usart.h>
#include <string.h>
#include <stdio.h>		//libreria sprintf()
#include <stdlib.h>		//librerias atoi(),atof()
#include "18F2550BOLT.h"
#include "PUERTO-SERIAL-BOLT.h"
#include "ONEWIRE-BOLT.h"

#define CONV const far rom char *
#define INCOMING_SIZE 220		// Tamao del arreglo de datos recibidos
#define OK			0
#define ERROR		1
#define NO_ANSWER	2
#define SLEEP		while(1){}

// Prototipo de funciones
void rx_handler (void);											// Prototipo de ISR USART
void initConfiguration(void);									// Configuraciones iniciales
void writeLCDmsg(const far rom char * message);					// Escribe una cadena en el LCD
void commandProcess(void);										// Funcion que procesa los comandos
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout);
																// Envia los comandos AT al modulo
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD);	// Escribe el estado del proceso de conexion
void sendATdata(const far rom char *data);						// Envia datos al modulo
unsigned char conect2SERVER(void);								// Funcion que conecta el modulo al servidor
void noConection(void);											// No es posible establecer la conexion

// Variables globales
volatile static char serialIncomingData[INCOMING_SIZE];			// Datos provenientes del puerto serie
volatile static unsigned char INDEX=0, INTERRUPT=0;				// Variables para el control de la interrupcion

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 		// Vector de reset
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code rx_interrupt = 0x000808					// Vector de interrupcion de alta prioridad
void rx_int (void)
{
	_asm goto rx_handler _endasm
}

#pragma code

#pragma interrupt rx_handler	// Funcion para el manejo de la interrupcion 
void rx_handler (void)
{
	serialIncomingData[INDEX++] = ReadUSART();	// Lee el buffer de entrada del puerto seria del pic
	PIR1bits.RCIF = 0;							// Limpia la bandera de interrupcion
	INTERRUPT=1;								// La interrupcion ha ocurrido
}
 
void main()						// Funcion principal
{
	unsigned char STATUS;		// Variable de control para monitorear el estado de la conexion
	initConfiguration();		// Configuracines iniciales 
	STATUS=conect2SERVER();		// Conecta el sitema BOLTd con un servidor remoto
	if(STATUS==OK)				// Si la conexion se ha establecido
		commandProcess();		// Procesa los comandos que envia el servidor
	else						// Si la conexion no se ha establecido 
		noConection();			// Envia un mensaje de error
} 

void noConection(void)						// Si la conexion no se ha establecido
{											// Envia un mensaje de error a traves del LCD
	writeLCDmsg((CONV)"  No se puede   ");	// del sistema BOLT
	delay_ms(800);
	writeLCDmsg((CONV)"   crear  el    ");
	delay_ms(800);
	writeLCDmsg((CONV)"servidor TCP/IP ");
	delay_ms(800);
	writeLCDmsg((CONV)"  Error TCP/IP  ");
	SLEEP;									// Loop infinito.
}

//
//	Funcion que envia todos los comandos AT necesarion para establecer la comunicacion con un 
//	servidor remoto a traves de el modulo, se requiere de conocer la IP y el 
//	numero de puerto del servidor al que se requiere conectar. Ademas monitorea constantemente
//	el estado de la conexion, si no es posible establecerla debido a que alguno de los comandos
//	no puede procesarse la funcion retorna un valor de ERROR.
//
unsigned char conect2SERVER(void)
{
	unsigned char STATUS;
	writeLCDmsg((CONV)"  BOLT-MSP8266  ");
	delay_ms(1000);
	writeLCDmsg((CONV)"   Aplicacion   ");
	delay_ms(1000);
	writeLCDmsg((CONV)"     TCP/IP     ");
	delay_ms(1000);
	writeLCDmsg((CONV)"Servidor TCP/IP ");
	delay_ms(1000);

	STATUS=sendATcommand((CONV)"AT",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"       AT       ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CIPCLOSE=0",(CONV)"",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CIPCLOSE   ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CIPSERVER=0",(CONV)"",1,1);
	sendSTATUS2LCD(STATUS,(CONV)" AT+CIPSERVER  ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CWJAP?",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CWJAP?    ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CIPMUX=1",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIPMUX    ");
	if(STATUS!=OK) return ERROR;

	STATUS=sendATcommand((CONV)"AT+CIFSR",(CONV)"OK",1,1);
	sendSTATUS2LCD(STATUS,(CONV)"   AT+CIFSR     ");
	if(STATUS!=OK) return ERROR;



	STATUS=sendATcommand((CONV)"AT+CIPSERVER=1,"SERVER_PORT,(CONV)"OK",2,2);
	sendSTATUS2LCD(STATUS,(CONV)"  AT+CIPSERVER  ");
	if(STATUS!=OK) return ERROR;

	writeLCDmsg((CONV)"Servidor online!");
	delay_ms(2000);
	return OK;		
}

void initConfiguration(void)	// Configuraciones iniciales para el sistema Bolt, cofigura el LCD, los puertos, el relevador
{								// y el puerto serie en hardware
	int initIndex;			// Variable local para limpiar el arreglo que contendra los datos recibidos
	init_bolt();			
	InitLCD();
	
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_ON &
	USART_ASYNCH_MODE & USART_EIGHT_BIT &
	USART_CONT_RX & USART_BRGH_HIGH, 25);
		
	RCONbits.IPEN = 1;		// Habilita prioridad en las interrupciones
	IPR1bits.RCIP = 1;		// Alta prioridad para la interrupcion por llegada de datos
	INTCONbits.GIEH = 1;	// Habilita todas las interrupciones de alta prioridad
	for(initIndex=0;initIndex<INCOMING_SIZE;initIndex++) serialIncomingData[initIndex]=0;
}

void sendATdata(const far rom char *data)	// Envia los datos al modulo  
{
	int i;
	for(i=0; i<strlenpgm(data); i++) 
	{
		while(BusyUSART());				// Espera a que el puerto serie este desocupado
		putcUSART(data[i]);				// Escribe cada uno de los caracteres al puerto serie
	}
	while(BusyUSART());
	putcUSART('\r');					// Retorno de carro
	while(BusyUSART());
	putcUSART((char)10);
	delay_ms(200);
}

void writeLCDmsg(const far rom char * message)		// Escribe la cadena "message" al LCD
{
	unsigned char i;
	ClearScreen();							// Limpia la pantalla
	GotoXY(0,0);							// Ve a la posicion (0,0)
	for(i=0; i<8; i++) PutLCD(message[i]);	// Imprime cada uno de los caracteres de la cadena del argumento
	GotoXY(1,0);
	for(i=8; i<strlenpgm(message);i++) PutLCD(message[i]);
}

//
//	Funcion que procesa los mensajes provenientes del servidor, si es un comando reconocido devuelve un mensaje 
//	indicando el efecto del comando.
//
void commandProcess(void)		
{
	int k;
	float temperatura;
	int entero, decimal;
	unsigned char STATUS;
	char on[]=		"enciende relevador";	// Cadenas de caracteres para el reconocimiento de comandodos
	char off[]=		"apaga relevador";		
	char cierra[]=	"cierra conexion";
	char tempe[]=	"lee temperatura";
	char temp[20];
	INDEX=0; 
	INTERRUPT=0;
	while(1)
	{
		if(INTERRUPT)						// Si la interrupcion ha ocurrido se procesan los datos.
		{
			if(strstr(serialIncomingData,on)) 
			{
				RELE=1;
				writeLCDmsg((CONV)"  Relevador ON  ");
				STATUS=sendATcommand((CONV)"AT+CIPSEND=0,21",(CONV)">",2,2);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador encendido\n\r!");
					//while(BusyUSART());
					//putcUSART((char)26);
					delay_ms(1000);
				}
			}
			else if(strstr(serialIncomingData,off)) 
			{
				RELE=0;
				writeLCDmsg((CONV)" Relevador OFF  ");
				STATUS=sendATcommand((CONV)"AT+CIPSEND=0,18",(CONV)">",2,2);
				if(STATUS==OK)
				{
					sendATdata((CONV)"Relevador apagado!\n\r");
					//while(BusyUSART());
					//putcUSART((char)26);
					delay_ms(1000);
				}
			}
			else if(strstr(serialIncomingData,tempe))
			{
				temperatura=ds1820_read();
				entero=temperatura;						// Parte entera
				decimal=(temperatura-entero)*10;		// Parte decimal
				sprintf(temp,(CONV)"Temp: %3d.%d C  \n\r",entero,decimal);		// Convierte a una cadena de caracteres
				ClearScreen();
				GotoXY(0,0);
				for(k=0; k<8; k++) PutLCD(temp[k]);
				GotoXY(1,0);
				for(k=8; k<11;k++) PutLCD(temp[k]);
				PutLCD(32);							//espacio
				PutLCD(214);						//simbolo de grados
				PutLCD('C');
				PutLCD(32);							//espacio
				PutLCD(32);							//espacio
				STATUS=sendATcommand((CONV)"AT+CIPSEND=0,17",(CONV)">",2,2);
				if(STATUS==OK)
				{
					for(k=0; k<strlen(temp); k++) 
					{
						while(BusyUSART());
						putcUSART(temp[k]);
					}
					while(BusyUSART());
					putcUSART('\r');
					//delay_ms(200);
					while(BusyUSART());
					putcUSART((char)10);
					delay_ms(1000);
				}
			}
			else if(strstr(serialIncomingData,cierra))
			{
				STATUS=sendATcommand((CONV)"AT+CIPCLOSE=0",(CONV)"OK",3,2);		// Cierra la conexion TCP/IP
				if(STATUS==OK)	writeLCDmsg((CONV)"Conexion cerrada");
				SLEEP;
			}		
			else 
			{
				STATUS=sendATcommand((CONV)"AT+CIPSEND=0,22",(CONV)">",2,2);		// Si el comando no se reconoce
				if(STATUS==OK)														// se envia el mensasaje 
				{																	// "No se reconoce comando"
					sendATdata((CONV)"No se reconoce comando"); //No se reconoce comando
					//while(BusyUSART());
					//putcUSART((char)26);
					delay_ms(1000);
				}
				writeLCDmsg((CONV)"Ninguna accion  ");
			}
			INDEX=0;
			INTERRUPT=0;
			for(k=0;k<INCOMING_SIZE;k++) serialIncomingData[k]=0;
		}
		delay_ms(1000);
	}
}

//	
//	Funcion que envia un comando AT al modulo, recibe el comando AT, la respuesta esperada, el retardo para el
//	procesamiento del comando AT por parte del modulo, y un tiempo limite, si el modulo no responde en 
//	el lapso de tiempo indicado por estr numero la funcion retorna el valor NO_ANSWER
//
unsigned char sendATcommand(const far rom char * command, const far rom char * answer, unsigned int delay, unsigned int timeout)
{
	unsigned long int time;
	int i;
	for(i=0; i<strlenpgm(command); i++) 
	{
		while(BusyUSART());		// Espera a que este disponible el modulo USART
		putcUSART(command[i]);	// envia cada uno de los caracteres del comando AT
	}
	while(BusyUSART());
	putcUSART('\r');			// equivale a pulsar ENTER
	while(BusyUSART());
	putcUSART((char)10);		
	delay_ms(delay*1000);
	for(time=0;time<(timeout*200000);time++)
	{
		if(INTERRUPT)
		{
			if(strstrrampgm(serialIncomingData,answer)) 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return OK;
			}
			else 
			{	
				INDEX=0;
				INTERRUPT=0;
				for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
				return ERROR;
			}
		}
		else continue;
	}
	INDEX=0;
	INTERRUPT=0;
	for(i=0;i<INCOMING_SIZE;i++) serialIncomingData[i]=0;
	return NO_ANSWER;
}

//
//	Funcion que envia un mensaje en el LCD indicando si un comando AT ha sido procesado correctamente
//
void sendSTATUS2LCD(unsigned char STATUS, const far rom char * message2LCD)
{
	writeLCDmsg(message2LCD);
	delay_ms(500);
	switch(STATUS){
	case OK:{ 
		writeLCDmsg((CONV)"       OK       ");
		delay_ms(500);
		break;}
	case ERROR:{
		writeLCDmsg((CONV)"      Error     ");
		delay_ms(500);
		break;}
	case NO_ANSWER:{
		writeLCDmsg((CONV)" Sin respuesta  ");
		delay_ms(500);
		break;}
	default: 
		break;}
} 



